/**
    MyXslt v0.2

    MyXslt is a Javascript tool using browser's built-in XSLT capacity to transform XML to HTML.

    Usage example:
    Xml.Config.useCache = false; // optional, only need to set once
    Xml.Config.logging = true; // optional, turn on logging if browser supports
    Xml.transform(someXml, "path/some.xslt", someHtmlElementToHoldResult);

    For optimal performance in production, set Xml.Config.useCache to true and
    Xml.Config.logging to false.

    It is modified and enhanced from source code in the excellent Javascript book
    "JavaScript: The Definitive Guide", 5th Edition, by David Flanagan (ISBN: 0596101996)
    Ref: http://www.davidflanagan.com/javascript5/display.php?n=xml.js&f=21/xml.js
    
    Special thanks to David Flanagan for giving me the permission to release it
    under MIT License.

    Thanks Aidan Garnish for the XHR cache buster solution:
    http://www.aidangarnish.net/post/XmlHttpRequest-Cache-Busters.aspx

    Updated: May 3, 2011 by Marshal

    -----------------------------------------------------------------------
    MIT License

    Copyright (c) 2011 Marshal

    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject to
    the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
    LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
    OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

// Make sure we haven't already been loaded
var Xml;
if (Xml && (typeof Xml !== "object" || Xml.Name)) {
    throw new Error("Namespace 'Xml' already exists");
}

// Create our namespace, and specify some meta-information
Xml = {};
Xml.Name = "Xml";     // The name of this namespace
Xml.Version = 1.0;    // The version of this namespace

// Configuration. Generally in production/QA, turn useCache on, while during development false (default)
Xml.Config = {useCache: false, logging: false};

/**
 * This Xml.Transformer class encapsulates an XSL stylesheet.
 * If the stylesheet parameter is a URL, we load it. Otherwise, we assume it is an appropriate DOM Document.
 */
Xml.Transformer = function(stylesheet) {
    if (typeof stylesheet === "string") { // Load the stylesheet if necessary
        stylesheet = Xml.load(stylesheet);
    }
    this.stylesheet = stylesheet; // store the stylesheet Document in this.stylesheet
    if (typeof XSLTProcessor !== "undefined") { // Mozilla
        this.processor = new XSLTProcessor(); // create an XSLTProcessor object
        this.processor.importStylesheet(this.stylesheet); // and import the stylesheet
    } // IE has NO such processor
};

/**
 * Synchronously load the Xml document and return it as a Document object
 * xml can be a xml string, url, or Document.
 */
Xml.load = function(xml) {
    var xmlDoc;
    if (typeof xml === "string") {
        if (/^\s*</.test(xml)) { // xml in string
            if (window.DOMParser) { // Mozilla
                xmlDoc = new DOMParser().parseFromString(xml, "text/xml");
            } else { // IE
                xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                xmlDoc.async = "false";
                xmlDoc.loadXML(xml);
            }
        } else { // Xml in url
            try {
                if (!this.xhttp && window.XMLHttpRequest) { // XHR not initialized && Mozilla/Safari/IE 7+, etc
                    this.xhttp = new XMLHttpRequest(); // Use native XMLHttpRequest object
                }
                // If environment is production, we can utilize XHR caching feature. Otherwise, eg. in testing,
                // the stylesheet is always changing, disable the cache by random param as "POST" is not allowed:
                this.xhttp.open("GET", Xml.Config.useCache ? xml : xml + "?bustCache=" + Math.random(), false);
                this.xhttp.send();
                // xhttp.responseXML.xml == "" meaning testing with local files:
                xmlDoc = this.xhttp.responseXML.xml === "" ? Xml.load(this.xhttp.responseText) : this.xhttp.responseXML;
                if (Xml.Config.logging) {
                    Xml.log("Loaded XSLT/XML:", this.xhttp.responseText);
                }
            } catch (e) { // mostly IE 5/6, or IE 7+ but loading local file: Access Denied
                if (window.ActiveXObject) { // IE, so try ActiveX:
                    this.xhttp = new ActiveXObject("Microsoft.XMLHTTP");
                    this.xhttp.open("GET", xml, false);
                    this.xhttp.send();
                    if (Xml.Config.logging) {
                        Xml.log("Loaded XSLT/XML:", this.xhttp.responseText);
                    }
                    // Local file's responseXML.xml would be blank, so recursively parse the responseText to Document:
                    xmlDoc = Xml.load(this.xhttp.responseText); // If not local file, this wouldn't harm either
                }
            }
        }
    } else { // assume it's a valid Document
        xmlDoc = xml;
    }

    return xmlDoc;
};

/**
 * This is the transform() method of the Xml.Transformer class.
 * It transforms the specified xml node using the encapsulated stylesheet.
 * The results of the transformation are assumed to be HTML and are used to replace the content of the specified element.
 */
Xml.Transformer.prototype.transform = function(node, element) {
    if (typeof element === "string") { // If element is specified by id, look it up
        element = document.getElementById(element);
    }

    if (this.processor) { // If we've created an XSLTProcessor (i.e. in Mozilla), use it.
        // Transform the node into a DOM DocumentFragment:
        var fragment = this.processor.transformToFragment(node, document);
        element.innerHTML = ""; // Erase the existing content of element
        element.appendChild(fragment); // And insert the transformed nodes
    } else if ("transformNode" in node) { // If the node has a transformNode() function (in IE), use that.
        element.innerHTML = node.transformNode(this.stylesheet); // Note that transformNode() returns a string
    } else { // Otherwise, we're out of luck.
        throw "XSLT is not supported in this browser";
    }

    if (Xml.Config.logging) {
        Xml.log("Transformed to:", element.innerHTML);
    }
};

/**
 * xml - xml (url/string/document)
 * stylesheet - stylesheet (url/string/document)
 * element - element that will hold the transformed xml
 */
Xml.transform = function(xml, stylesheet, element) {
    var xmlDoc = Xml.load(xml);
    // If the stylesheet has not been loaded or has changed or cache disabled:
    if (!this.transformer || this.stylesheet !== stylesheet || !Xml.Config.useCache) {
        this.transformer = new Xml.Transformer(stylesheet);
    } // Otherwise reuse the transformer created before

    this.transformer.transform(xmlDoc, element);
};

Xml.log = function(info, message) {
    if (window.console && window.console.log) {
        window.console.log(info + "\n" + message);
    }
};