/**
 * Unit tests of myxslt using jQuery QUnit.
 * Tested in IE8 and FF4 with local xslt files. Not tested with xslt deployed to server such as IIS, etc.
 *
 * Marshal 2011-4-27
 */
$(function() {
    QUnit.log = function(message) {
        if (window.console && window.console.log) {
            window.console.log(message);
        }
    }

    var url = document.location.href, // used to get current dir from URL
        pwd = url.substring(8, url.lastIndexOf("/") + 1).replace(/\//g,"\\"), // eg. file:///C:/myxslt/test/unitTest.html -> C:\\myxslt\\test
        BAT = pwd + "changeContent.bat", // DOS batch to swap xslt content
        XSLT= pwd + "test2.xslt"; // XSLT file whose content will be verified

    var TEST_XSLT_CONTENT = /.*Original XSLT - No result.*/, // test.xslt verification regex
        TEST2_XSLT_CONTENT = /.*Another XSLT used.*/; // test2.xslt verification regex

    module("Directly read from files", {
        setup: function() { Xml.Config.useCache = false; Xml.Config.logging = true; }
    });

    // Test1
    test("xml=sample.xml, xslt=test.xslt", function() {
        equal($("#case1")[0].innerHTML, "TestCase1: xml (url/file) and xslt (url/file)."); // original

        Xml.transform("sample.xml", "test.xslt", $("#case1")[0]);

        // in FF, the transformed is like "\n \n Original XSLT - No result\n \n \n":
        ok(/^[\s\t\n]*Original XSLT - No result[\s\t\n]*/.test($($("#case1")[0]).text())); // transformed
    });

    module("Read file using ajax");

    // Test2
    // Tell QUnit that expect 3 assertions to run in async mode
    asyncTest("xml=sample2.xml, xslt=test.xslt", 3, function() {
        // The test is automatically paused here. Check original first:
        equal($("#case2")[0].innerHTML, "TestCase2: xml with declaration (url/file) through .ajax, xslt (url/file).");

        $.ajax({
            url: "sample2.xml",
            type: "GET",
            dataType: "xml",
            complete: function(jqXhr) {
                // Even specify dataType is xml, IE returns responseText. FireFox does the right thing:
                Xml.transform(jqXhr.responseXML ? jqXhr.responseXML : jqXhr.responseText, "test.xslt", $("#case2")[0]);
                // in FF, the transformed is like "\n \n Original XSLT - No result\n \n \n":
                ok(TEST_XSLT_CONTENT.test($("#case2")[0].innerHTML));
                equal(Xml.Config.useCache, false);

                start(); // resume QUnit
            }
        });
    });

    /**
     * Create a common test function.
     * xml: xml to be transformed
     * xslt: xslt
     * element: element to hold the transformed HTML content
     * msg: QUnit test message
     * originalContent: content before transformation
     * newContentRegex: content verification regex after transformation
     */
    var testCase = function(xml, xslt, element, msg, originalContent, newContentRegex) {
        asyncTest(msg + ", xml=" + xml + ", xslt=" + xslt, function() {
            equal($(element)[0].innerHTML, originalContent);

            $.get(xml).complete(function (jqXhr) {
                Xml.transform(jqXhr.responseXML ? jqXhr.responseXML : jqXhr.responseText, xslt, $(element)[0]);
                ok(newContentRegex.test($(element)[0].innerHTML), "Expect regex=" + newContentRegex + ", Actual=" + $(element)[0].innerHTML);

                start(); // resume QUnit
            });
        });
    };

    // Test3
    testCase("sample.xml", "test2.xslt", "#case3", "Use .get()",
        "TestCase3: xml (url/file) through .get, xslt (url/file).", TEST2_XSLT_CONTENT);

    module("Xml.Config.useCache, change content of test2.xslt to test.xslt", {
        setup: function() {
            Xml.Config.useCache = true;
            var xml = "sample.xml",
                xslt = "test2.xslt",
                element = "#case3",
                newContentRegex = TEST2_XSLT_CONTENT;
            // Run a tranformation prior to Test4 to see if useCache works
            $.ajax({
                url: xml,
                type: "GET",
                async: false
            }).complete(function (jqXhr) {
                QUnit.log("Setup start...");
                Xml.transform(jqXhr.responseXML ? jqXhr.responseXML : jqXhr.responseText, xslt, $(element)[0]);
                equal(Xml.Config.useCache, true, "setup: useCache turned on");
                ok(newContentRegex.test($(element)[0].innerHTML), "setup: Expect regex=" + newContentRegex + ", Actual=" + $(element)[0].innerHTML);
                QUnit.log("Setup done, ready for Test4");
            });

            stop();
            callLocalExe([pwd]); // change the content of test2.xslt to test.xslt
        },
        teardown: function() {
            stop();
            callLocalExe([pwd, "-r"]); // revert the content of test2.xslt back
        }
    });

    /**
     * Because useCache, the content of test2.xslt is now the content of test.xslt, and
     * the result should still be TEST2_XSLT_CONTENT rather than TEST_XSLT_CONTENT.
     * However when test in browsers locally, it's not cached at all.
     * This is totally different when myxslt is deployed to server, eg. SharePoint 2007
     */
    // Test4
    testCase("sample.xml", "test2.xslt", "#case4", "No local cache in IE & Firefox",
            "TestCase4: TestCase3 + useCache=true.", TEST_XSLT_CONTENT);

    module("Xml.Config.useCache, change xslt file should reflect the new one");

    // Test5
    testCase("sample.xml", "test.xslt", "#case4", "Test that even cache on, when xslt changes, it uses the updated one",
        "TestCase4: TestCase3 + useCache=true.", TEST_XSLT_CONTENT);

    function callLocalExe(args) {
        if ($.browser.mozilla) { // FF, https://developer.mozilla.org/en/JavaScript_code_modules/NetUtil.jsm
            netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
            // create an nsILocalFile for the executable
            var file = Components.classes["@mozilla.org/file/local;1"]
                                 .createInstance(Components.interfaces.nsILocalFile);
            file.initWithPath(BAT);

            // create an nsIProcess
            var process = Components.classes["@mozilla.org/process/util;1"]
                                    .createInstance(Components.interfaces.nsIProcess);

            process.init(file);

            // Run the process.
            // The first param is true, calling thread will be blocked until called process terminates.
            // Second and third params are used to pass command-line arguments to the process.
            process.run(true, args, args.length);
            QUnit.log(args.length <= 1 ? "test2.xslt content now changed to test.xslt" : "test2.xslt restored");

            // Now verify the file content has been changed
            file.initWithPath(XSLT);
            // Components.utils.import("resource://gre/modules/NetUtil.jsm"); // Compilation error in IE
            Components.utils["import"]("resource://gre/modules/NetUtil.jsm");
            // still cannot use string URL directly in FF4, use nsiFile instead:
            NetUtil.asyncFetch(file, function(inputStream, status) {
                // Warning: without the following, readInputStreamToString will freeze:
                netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
                if (!Components.isSuccessCode(status)) {
                    alert("Error read file: " + xslt);
                    return;
                }

                // The file data is contained within inputStream, can read it into a string
                var data = NetUtil.readInputStreamToString(inputStream, inputStream.available());
                if (args.length <= 1) {
                    ok(TEST_XSLT_CONTENT.test(data), "File content verified");
                } else { // revert
                    ok(TEST2_XSLT_CONTENT.test(data), "File content verified");
                }
                
                start(); // till now, can resume the testing
            });

        } else if ($.browser.msie) { // IE, http://msdn.microsoft.com/en-us/library/hwfw5c59%28v=vs.85%29.aspx
            var shell = new ActiveXObject("WScript.Shell");
            shell.run(BAT + " " + args, 0, true); // 0 - hide the window, true - wait till finish (synchronous)
            QUnit.log(args.length <= 1 ? "test2.xslt content now changed to test.xslt" : "test2.xslt restored");

            var fo = new ActiveXObject("Scripting.FileSystemObject"),
                f = fo.GetFile(XSLT),
                s = f.OpenAsTextStream(1, 0), // 1 - ForReading, 0 - Open the file as ASCII
                found = false;

            while (!found && !s.AtEndOfStream) { // read until find a match or hit the end of the file
                if (args.length <= 1) {
                    found = TEST_XSLT_CONTENT.test(s.ReadLine());
                } else { // revert
                    found = TEST2_XSLT_CONTENT.test(s.ReadLine());
                }
            }

            s.Close(); // close the stream
            ok(found, "File content verified");

            start();
        }
    }
});

